use telers::{
    errors::SessionErrorKind,
    methods::GetChatMember,
    types::{ChatMember, Message},
    Bot,
};

#[derive(Clone, Default, Debug)]
pub enum TargetUser {
    Id(i64),
    Reply(Message),
    CurrentMessage(Message),
    #[default]
    None,
}

impl TargetUser {
    pub fn exist(&self) -> bool {
        match self {
            Self::Id(_id) => true,
            Self::Reply(msg) => {
                if let Some(replied_msg) = msg.reply_to_message() {
                    return replied_msg.from().map(|user| user.id).is_some();
                } else {
                    false
                }
            }
            Self::CurrentMessage(msg) => msg.from().map(|user| user.id).is_some(),
            Self::None => false,
        }
    }

    pub fn get_id(&self) -> Option<i64> {
        match self {
            Self::Id(id) => Some(*id),
            Self::Reply(msg) => {
                if let Some(replied_msg) = msg.reply_to_message() {
                    replied_msg.from().map(|user| user.id)
                } else {
                    None
                }
            }
            Self::CurrentMessage(msg) => msg.from().map(|user| user.id),
            Self::None => None,
        }
    }
    pub async fn get_user_name(&self, bot: &Bot, msg: &Message) -> Option<String> {
        match self {
            Self::Id(id) => {
                let get_chat_member_result: Result<ChatMember, SessionErrorKind> =
                    bot.send(GetChatMember::new(msg.chat().id(), *id)).await;

                if let Ok(member_kind) = get_chat_member_result {
                    match member_kind {
                        ChatMember::Owner(member) => member.user.username,
                        ChatMember::Administrator(member) => member.user.username,
                        ChatMember::Member(member) => member.user.username,
                        ChatMember::Restricted(member) => member.user.username,
                        ChatMember::Left(member) => member.user.username,
                        ChatMember::Banned(member) => member.user.username,
                    }
                } else {
                    None
                }
            }
            Self::Reply(msg) => {
                if let Some(replied_msg) = msg.reply_to_message() {
                    replied_msg.from().unwrap().username.clone()
                } else {
                    None
                }
            }
            Self::CurrentMessage(msg) => {
                if let Some(replied_msg) = msg.from() {
                    replied_msg.username.clone()
                } else {
                    None
                }
            }
            Self::None => None,
        }
    }
}
