use core::convert::Into;
use telers::{
    enums::ParseMode,
    errors::session::ErrorKind,
    methods::SendMessage,
    types::{Message, ReplyParameters},
    Bot,
};

#[derive(Clone, Debug)]
pub struct MessageSender {
    message: SendMessage,
}

impl MessageSender {
    pub fn builder(group_id: i64) -> MessageSenderBuilder {
        MessageSenderBuilder {
            chat_id: group_id,
            send_message: SendMessage::new(group_id, ""),
        }
    }
    pub async fn send(self, bot: &Bot) -> Result<Message, ErrorKind> {
        bot.send(self.message).await
    }
}

#[derive(Clone, Debug)]
pub struct MessageSenderBuilder {
    chat_id: i64,
    send_message: SendMessage,
}

#[allow(dead_code)]
impl MessageSenderBuilder {
    pub fn parse_mode(mut self, parse_mode: ParseMode) -> Self {
        self.send_message = self.send_message.parse_mode(parse_mode);
        self
    }
    pub fn group_id(mut self, val: i64) -> Self {
        self.chat_id = val;
        self.send_message = self.send_message.chat_id(val);
        self
    }
    pub fn text(mut self, val: impl Into<String>) -> Self {
        self.send_message = self.send_message.text(val);
        self
    }
    pub fn disable_notification(mut self, val: bool) -> Self {
        self.send_message = self.send_message.disable_notification(val);
        self
    }
    pub fn reply_to(mut self, msg_id: i64) -> Self {
        let reply = ReplyParameters::new(msg_id).chat_id(self.chat_id);
        self.send_message = self.send_message.reply_parameters(reply);
        self
    }

    pub fn set_parse_mode(&mut self, parse_mode: ParseMode) {
        self.send_message = self.send_message.clone().parse_mode(parse_mode);
    }
    pub fn set_group_id(&mut self, val: i64) {
        self.chat_id = val;
        self.send_message = self.send_message.clone().chat_id(val);
    }
    pub fn set_text(&mut self, val: impl Into<String>) {
        self.send_message = self.send_message.clone().text(val);
    }
    pub fn set_disable_notification(&mut self, val: bool) {
        self.send_message = self.send_message.clone().disable_notification(val);
    }
    pub fn set_reply_to(&mut self, msg_id: i64) {
        let reply = ReplyParameters::new(msg_id).chat_id(self.chat_id);
        self.send_message = self.send_message.clone().reply_parameters(reply);
    }

    pub fn build(self) -> MessageSender {
        MessageSender {
            message: self.send_message,
        }
    }
}
